<?php

namespace Drupal\cloudconvert_media_thumbnail\EventSubscriber;

use Drupal\cloudconvert\Event\CloudConvertFinishEvent;
use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;
use Drupal\Core\Utility\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class MediaThumbnailSubscriber
 *
 * @package Drupal\cloudconvert\EventSubscriber
 */
class MediaThumbnailSubscriber implements EventSubscriberInterface, ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * Subscribe to KernelEvents::REQUEST events and redirect if site is currently
   * in maintenance mode.
   */

  public function __construct(EntityTypeManagerInterface $entityTypeManager, Token $token) {
    $this->entityTypeManager = $entityTypeManager;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  static function getSubscribedEvents() {
    $events[CloudConvertFinishEvent::FINISH][] = ['finishMediaTask'];
    return $events;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('token')
    );
  }

  /**
   * The finish media task event handler to set the thumbnail for media_thumbnail
   * bundles.
   *
   * @param \Drupal\cloudconvert\Event\CloudConvertFinishEvent $event
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function finishMediaTask(CloudConvertFinishEvent $event) {
    $cloudConvertTask = $event->getCloudConvertTask();

    if ($cloudConvertTask->bundle() != 'media_thumbnail') {
      return;
    }

    /** @var \Drupal\media\MediaInterface $media */
    $mediaItems = $cloudConvertTask->get('field_media')->referencedEntities();
    $media = reset($mediaItems);
    $fileStorage = $this->entityTypeManager->getStorage('file');

    $fileInfo = pathinfo($event->getResult());
    $thumbnailDirectory = $this->getThumbnailDirectory($media);
    $thumbnailUri = file_unmanaged_move($event->getResult(), $thumbnailDirectory . '/' . $fileInfo['basename']);

    /** @var \Drupal\file\FileInterface $file */
    $file = $fileStorage->create();
    $file->setFileUri($thumbnailUri);
    if ($owner = $media->getOwner()) {
      $file->setOwner($owner);
    }
    $file->setPermanent();
    $file->save();

    $this->setThumbnail($media, $file);
  }

  /**
   * Get the thumbnail directory.
   *
   * @param \Drupal\media\MediaInterface $media
   *
   * @return string
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   */
  private function getThumbnailDirectory(MediaInterface $media) {
    $mediaTypeId = $media->bundle();
    $mediaTypeStorage = $this->entityTypeManager->getStorage('media_type');
    /** @var \Drupal\media\MediaTypeInterface $mediaType */
    $mediaType = $mediaTypeStorage->load($mediaTypeId);
    $fieldName = $mediaType->getSource()->getConfiguration()['source_field'];
    $fieldDefinition = $media->getFieldDefinition($fieldName);
    $targetFieldSettings = $fieldDefinition->getSettings();

    $baseThumbnailDirectory = trim($targetFieldSettings['file_directory'], '/') . '/thumbnail';
    $baseThumbnailDirectory = PlainTextOutput::renderFromHtml($this->token->replace($baseThumbnailDirectory));

    $thumbnailDirectory = $targetFieldSettings['uri_scheme'] . '://' . $baseThumbnailDirectory;
    file_prepare_directory($thumbnailDirectory, FILE_CREATE_DIRECTORY);
    return $thumbnailDirectory;
  }

  /**
   * Set the new file as the thumbnail.
   *
   * @param \Drupal\media\MediaInterface $media
   * @param \Drupal\file\FileInterface $file
   */
  private function setThumbnail(MediaInterface $media, FileInterface $file) {
    $media->thumbnail->target_id = $file->id();

    // Set the thumbnail alt.
    $mediaSource = $media->getSource();
    $plugin_definition = $mediaSource->getPluginDefinition();
    if (!empty($plugin_definition['thumbnail_alt_metadata_attribute'])) {
      $media->thumbnail->alt = $mediaSource->getMetadata($media, $plugin_definition['thumbnail_alt_metadata_attribute']);
    }
    else {
      $media->thumbnail->alt = $this->t('Thumbnail', [], ['langcode' => $media->langcode->value]);
    }

    // Set the thumbnail title.
    if (!empty($plugin_definition['thumbnail_title_metadata_attribute'])) {
      $media->thumbnail->title = $mediaSource->getMetadata($media, $plugin_definition['thumbnail_title_metadata_attribute']);
    }
    else {
      $media->thumbnail->title = $media->label();
    }
    $media->save();
  }
}
