<?php
/**
 * Created by PhpStorm.
 * User: jeffreybertoen
 * Date: 06-02-18
 * Time: 19:53
 */

namespace Drupal\cloudconvert_media_thumbnail;

use Drupal\cloudconvert\CloudConvertProcessor;
use Drupal\cloudconvert\Entity\CloudConvertTaskInterface;
use Drupal\cloudconvert\Parameters;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Image\ImageFactory;
use Drupal\Core\Queue\QueueFactory;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class MediaThumbnailManager
 *
 * @package Drupal\cloudconvert
 */
class MediaThumbnailManager implements ContainerInjectionInterface, MediaThumbnailManagerInterface {

  /**
   * Cloud Convert Processor.
   *
   * @var \Drupal\cloudconvert\CloudConvertProcessor
   */
  protected $cloudConvertProcessor;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Queue Factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * Media Type Storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $mediaTypeStorage;

  /**
   * Output format to convert to.
   *
   * @var string
   */
  protected $outputFormat = 'jpg';

  /**
   * @inheritDoc
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, CloudConvertProcessor $cloudConvertProcessor, QueueFactory $queueFactory) {
    $this->entityTypeManager = $entityTypeManager;
    $this->cloudConvertProcessor = $cloudConvertProcessor;
    $this->queueFactory = $queueFactory;
    $this->mediaTypeStorage = $entityTypeManager->getStorage('media_type');
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container) {
    return new static (
      $container->get('entity_type.manager'),
      $container->get('cloudconvert.processor'),
      $container->get('queue')
    );
  }

  /**
   * @inheritDoc
   */
  public function queueThumbnailTask(MediaInterface $media) {
    /** @var FileInterface $file */
    $file = $this->getMediaFile($media);

    if (!$this->isThumbnailGenerationNeeded($media)) {
      return;
    }

    $cloudConvertTaskTypeStorage = $this->entityTypeManager->getStorage('cloudconvert_task_type');
    /** @var \Drupal\cloudconvert\Entity\CloudConvertTaskTypeInterface $cloudConvertTaskType */
    $cloudConvertTaskType = $cloudConvertTaskTypeStorage->load('media_thumbnail');
    $cloudConvertTask = $this->cloudConvertProcessor->createTask($cloudConvertTaskType, $file);
    $cloudConvertTask->set('field_media', $media);

    $cloudConvertTask->save();

    $inputFormat = pathinfo($file->getFilename(), PATHINFO_EXTENSION);
    $mode = $this->getMode($cloudConvertTask);

    if (empty($mode)) {
      return;
    }

    $parameters = new Parameters([
      'mode' => $mode,
      'inputformat' => $inputFormat,
      'converteroptions' => [
        'page_range' => '1-1',
        'thumbnail_size' => '1920x',
        'resize' => '1920x',
        'strip_metatags' => TRUE,
        'density' => '75',
      ],
    ]);

    if ($mode === 'convert') {
      $parameters->setOutputformat($this->outputFormat);
    }

    $this->cloudConvertProcessor->createStartQueueItem($cloudConvertTask, $parameters);
  }

  /**
   * Get the Media File.
   *
   * @param \Drupal\media\MediaInterface $media
   *
   * @return mixed
   */
  private function getMediaFile(MediaInterface $media) {
    $mediaType = $this->getMediaType($media);
    $fieldName = $this->getSourceFieldName($mediaType);

    /** @var \Drupal\file\FileInterface $file */
    return $media->get($fieldName)->entity;
  }

  /**
   * Get the Media Type.
   *
   * @param \Drupal\media\MediaInterface $media
   *
   * @return \Drupal\media\MediaTypeInterface
   */
  private function getMediaType(MediaInterface $media) {
    $mediaTypeId = $media->bundle();
    /** @var \Drupal\media\MediaTypeInterface $mediaType */
    $mediaType = $this->mediaTypeStorage->load($mediaTypeId);
    return $mediaType;
  }

  /**
   * Get the Source field name.
   *
   * @param \Drupal\media\MediaTypeInterface $mediaType
   *
   * @return string
   */
  private function getSourceFieldName(MediaTypeInterface $mediaType) {
    $mediaSource = $mediaType->getSource();
    $fieldDefinition = $mediaSource->getSourceFieldDefinition($mediaType);
    return $fieldDefinition->getName();
  }

  /**
   * @param \Drupal\media\MediaInterface $media
   *
   * @return bool
   */
  public function isThumbnailGenerationNeeded(MediaInterface $media) {
    $mediaType = $this->getMediaType($media);
    return $mediaType->getSource()->getPluginDefinition()['id'] !== 'image';
  }

  /**
   * Get the mode for the process.
   *
   * @param \Drupal\cloudconvert\Entity\CloudConvertTaskInterface $cloudConvertTask
   *
   * @return string
   */
  protected function getMode(CloudConvertTaskInterface $cloudConvertTask) {
    $file = $cloudConvertTask->getOriginalFile();
    $inputFormat = pathinfo($file->getFilename(), PATHINFO_EXTENSION);
    if ($this->validateImageConversion($inputFormat, $this->outputFormat)) {
      return 'convert';
    }
    elseif ($this->validateThumbnailConversion($file)) {
      return 'info';
    }

    return '';
  }

  /**
   * @inheritDoc
   */
  public function validateImageConversion($inputFormat, $outputFormat) {
    $result = $this->cloudConvertProcessor->getCloudConvertApi()
      ->get('/conversiontypes', [
        'inputformat' => $inputFormat,
        'outputformat' => $outputFormat,
      ]);

    return !empty($result);
  }

  /**
   * @inheritDoc
   */
  public function validateThumbnailConversion($inputFormat) {
    $result = $this->cloudConvertProcessor->getCloudConvertApi()
      ->get('/conversiontypes', [
        'inputformat' => $inputFormat,
      ]);

    if (empty($result)) {
      return FALSE;
    }

    foreach ($result as $conversionOption) {
      $options = array_keys($conversionOption['converteroptions']);
      if (in_array('thumbnail_format', $options)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function mediaSourceHasChanged(MediaInterface $media) {
    $mediaType = $this->getMediaType($media);
    $fieldName = $this->getSourceFieldName($mediaType);

    if (!isset($media->original)) {
      return FALSE;
    }

    $langcodes = array_keys($media->getTranslationLanguages());
    if ($langcodes === array_keys($media->original->getTranslationLanguages())) {
      // If the list of langcodes has changed, we need to save.
      return FALSE;
    }

    foreach ($langcodes as $langcode) {
      if(!$this->mediaSourceTranslationHasChanged($media, $fieldName, $langcode)) {
        continue;
      }

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Check if a translation value of the field is changed.
   *
   * @param \Drupal\media\MediaInterface $media
   * @param string $fieldName
   * @param string $langcode
   *
   * @return bool
   */
  protected function mediaSourceTranslationHasChanged(MediaInterface $media, $fieldName, $langcode) {
    $items = $media->getTranslation($langcode)
      ->get($fieldName)
      ->filterEmptyItems();
    $original_items = $media->original->getTranslation($langcode)
      ->get($fieldName)
      ->filterEmptyItems();
    // If the field items are not equal, we need to save.
    if ($items->equals($original_items)) {
      return FALSE;
    }

    return TRUE;
  }
}
