<?php

namespace Drupal\cloudconvert;

/**
 * Base class for Objects returned from the CloudConvert API
 *
 * @package CloudConvert
 * @category CloudConvert
 * @author Josias Montag <josias@montag.info>
 */
class ApiObject {

  /**
   * URL parameter.
   *
   * @var string
   */
  public $url;

  /**
   * Contains the API object.
   *
   * @var \Drupal\cloudconvert\Api
   */
  protected $api;

  /**
   * Contains the object data returned from the CloudConvert API
   *
   * @var array
   */
  protected $data = [];

  /**
   * Construct a new ApiObject instance
   *
   * @param Api $api
   * @param string $url The Object URL
   *
   * @throws Exceptions\InvalidParameterException If one parameter is missing
   *   or with bad value
   */
  public function __construct(Api $api, $url) {
    if (!isset($api)) {
      throw new Exceptions\InvalidParameterException("API parameter is not set");
    }
    if (!isset($url)) {
      throw new Exceptions\InvalidParameterException("Object URL parameter is not set");
    }
    $this->api = $api;
    $this->url = $url;
    return $this;
  }

  /**
   * Refresh Object Data
   *
   * @param \Drupal\cloudconvert\Parameters $parameters Parameters for refreshing the
   *   Object.
   *
   * @return \Drupal\cloudconvert\ApiObject
   */
  public function refresh(Parameters $parameters = NULL) {
    if (!isset($parameters)) {
      $parameters = new Parameters([]);
    }
    $this->data = $this->api->get($this->url, $parameters->getParameters(), FALSE);
    return $this;
  }

  /**
   * Access Object data via $object->prop->subprop
   *
   * @param string $name
   *
   * @return null|object
   */
  public function __get($name) {

    if (!is_array($this->data) || !array_key_exists($name, $this->data)) {
      return NULL;
    }

    if (is_object($this->data[$name])) {
      return $this->data[$name];
    }

    return self::arrayToObject($this->data[$name]);
  }

  /**
   * Converts multi dimensional arrays into objects
   *
   * @param array|object $d
   *
   * @return object
   */
  private static function arrayToObject($d) {
    if (is_array($d)) {
      /*
       * Return array converted to object
       * Using [__CLASS__, __METHOD__] (Magic constant)
       * for recursive call
       */
      return (object) array_map([__CLASS__, __METHOD__], $d);
    }
    return $d;
  }

  /**
   * Get the data array.
   *
   * @return array
   */
  public function getData() {
    return $this->data;
  }

  /**
   * Get the name of the object.
   *
   * @param $name
   *
   * @return null|object
   */
  public function get($name) {
    return $this->__get($name);
  }
}
