<?php

namespace Drupal\cloudconvert;

/**
 * Class Parameters
 *
 * This manages the parameters for use with CloudConvert requests.
 *
 * @package CloudConvert
 */
class Parameters {

  /**
   * API Key to authenticate with.
   *
   * @var string
   *   (required)
   */
  protected $apikey;

  /**
   * The format of the original file.
   *
   * @var string
   *   (required)
   */
  protected $inputformat;

  /**
   * The format to convert to.
   *
   * @var string
   *   Required for all modes, except info.
   */
  protected $outputformat;

  /**
   * The mode of the process.
   *
   * @var string
   *   (optional) Defaults to convert, can also be info, combine, archive and
   *   extract.
   */
  protected $mode;

  /**
   * Method of providing the input file.
   *
   * @var string
   *   (required)
   */
  protected $input;

  /**
   * Override the input filename.
   *
   * @var string
   *   (optional)
   */
  protected $filename;

  /**
   * A tag to identify the conversion, can also be an array.
   *
   * @var string|array
   *   (optional)
   */
  protected $tag;

  /**
   * Number of seconds before the conversion is cancelled.
   *
   * @var int
   *   (optional)
   */
  protected $timeout;

  /**
   * A preset of settings that is available to the account on Cloudconvert.com.
   *
   * @var string
   *   (optional)
   */
  protected $preset;

  /**
   * Options for the converter, specific to the chosen inputformat, outputformat
   * and mode.
   *
   * @var array
   *   (optional)
   */
  protected $converteroptions;

  /**
   * Send email notification after processing.
   *
   * @var bool
   *   (optional)
   */
  protected $email;

  /**
   * The output storage to save the file to.
   *
   * @var string
   *   (optional)
   */
  protected $output;

  /**
   * Parameter to wait till the processing is finished.
   *
   * @var bool
   *   (optional)
   */
  protected $wait;

  /**
   * Callback url to call when Cloudconvert.com is done processing.
   *
   * @var string
   *   (optional)
   */
  protected $callback;

  /**
   * Parameter to wait till the file is processed and downloaded.
   *
   * @var bool|string
   *   (optional)
   */
  protected $download;

  /**
   * Parameter to save the files on Cloudconvert.com.
   *
   * @var bool
   *   (optional)
   */
  protected $save;

  /**
   * Array containing the parameters for the process.
   *
   * @var array
   */
  protected $parameterList = [
    'apikey',
    'inputformat',
    'input',
    'file',
    'files',
    'filename',
    'tag',
    'outputformat',
    'converteroptions',
    'preset',
    'mode',
    'timeout',
    'email',
    'output',
    'callback',
    'wait',
    'download',
    'save',
  ];

  /**
   * Parameters constructor.
   *
   * @param array $parameters
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function __construct(array $parameters = []) {
    $parameterList = $this->parameterList;
    foreach ($parameters as $parameter => $value) {
      if (!in_array($parameter, $parameterList)) {
        continue;
      }
      $this->setParam($parameter, $value);
    }
  }

  /**
   * Get a parameter if it is valid.
   *
   * @param $parameter
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  protected function getParam($parameter) {
    $this->validateParameter($parameter);
    return isset($this->{$parameter}) ? $this->{$parameter} : NULL;
  }

  /**
   * Set the parameter if its valid.
   *
   * @param $name
   * @param $value
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  protected function setParam($name, $value) {
    $this->validateParameter($name);
    $this->{$name} = $value;
  }

  /**
   * Validate the given parameter.
   *
   * @param string $parameter
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  protected function validateParameter($parameter) {
    $parameterList = $this->parameterList;
    if (!in_array($parameter, $parameterList)) {
      throw new Exceptions\InvalidParameterException("Parameter is not valid.");
    }
  }

  /**
   * Get the API key parameter.
   *
   * @return string
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getApikey() {
    return $this->getParam('apikey');
  }

  /**
   * Set the API key parameter.
   *
   * @param string $apikey
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setApikey($apikey) {
    $this->setParam('apikey', $apikey);
  }

  /**
   * Get the inputformat parameter.
   *
   * @return string
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getInputformat() {
    return $this->getParam('inputformat');
  }

  /**
   * Set the inputformat parameter.
   *
   * @param string $inputformat
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setInputformat($inputformat) {
    $this->setParam('inputformat', $inputformat);
  }

  /**
   * Get the outputformat parameter.
   *
   * @return string
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getOutputformat() {
    return $this->getParam('outputformat');
  }

  /**
   * Set the outputformat parameter.
   *
   * @param string $outputformat
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setOutputformat($outputformat) {
    $this->setParam('outputformat', $outputformat);
  }

  /**
   * Get the mode parameter.
   *
   * @return string
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getMode() {
    return $this->getParam('mode');
  }

  /**
   * Set the mode parameter.
   *
   * @param string $mode
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setMode($mode) {
    $this->setParam('mode', $mode);
  }

  /**
   * Get the input parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getInput() {
    return $this->getParam('input');
  }

  /**
   * Set the input parameter.
   *
   * @param $input
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setInput($input) {
    $this->setParam('input', $input);
  }

  /**
   * Get the filename parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getFilename() {
    return $this->getParam('filename');
  }

  /**
   * Set the filename parameter.
   *
   * @param $filename
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setFilename($filename) {
    $this->setParam('filename', $filename);
  }

  /**
   * Get the tag parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getTag() {
    return $this->getParam('tag');
  }

  /**
   * Set the tag parameter.
   *
   * @param $tag
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setTag($tag) {
    $this->setParam('tag', $tag);
  }

  /**
   * Get the timeout parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getTimeout() {
    return $this->getParam('timeout');
  }

  /**
   * Set the timeout parameter.
   *
   * @param $timeout
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setTimeout($timeout) {
    $this->setParam('timeout', $timeout);
  }

  /**
   * Get the preset parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getPreset() {
    return $this->getParam('preset');
  }

  /**
   * Set the preset parameter.
   *
   * @param $preset
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setPreset($preset) {
    $this->setParam('preset', $preset);
  }

  /**
   * Get the convertoptions parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getConverteroptions() {
    return $this->getParam('converteroptions');
  }

  /**
   * Set the convertoptions parameters.
   *
   * @param $converteroptions
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setConverteroptions($converteroptions) {
    $this->setParam('converteroptions', $converteroptions);
  }

  /**
   * Get a single convert option.
   *
   * @param $option
   *
   * @return mixed
   */
  public function getConverteroption($option) {
    return $this->converteroptions[$option];
  }

  /**
   * Set a single convert option.
   *
   * @param $option
   * @param $setting
   */
  public function setConverteroption($option, $setting) {
    $this->converteroptions[$option] = $setting;
  }

  /**
   * Get the email parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getEmail() {
    return $this->getParam('email');
  }

  /**
   * Set the email parameter.
   *
   * @param $email
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setEmail($email) {
    $this->setParam('email', $email);
  }

  /**
   * Get the output parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getOutput() {
    return $this->getParam('output');
  }

  /**
   * Set the output parameter.
   *
   * @param $output
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setOutput($output) {
    $this->setParam('output', $output);
  }

  /**
   * Get the wait parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getWait() {
    return $this->getParam('wait');
  }

  /**
   * Set the wait parameter.
   *
   * @param $wait
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setWait($wait) {
    $this->setParam('wait', $wait);
  }

  /**
   * Get the callback parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getCallback() {
    return $this->getParam('callback');
  }

  /**
   * Set the callback parameter.
   *
   * @param $callback
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setCallback($callback) {
    $this->setParam('callback', $callback);
  }

  /**
   * Get the download parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getDownload() {
    return $this->getParam('download');
  }

  /**
   * Set the download parameter.
   *
   * @param $download
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setDownload($download) {
    $this->setParam('download', $download);
  }

  /**
   * Get the save parameter.
   *
   * @return mixed
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getSave() {
    return $this->getParam('save');
  }

  /**
   * Set the save parameter.
   *
   * @param $save
   *
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function setSave($save) {
    $this->setParam('save', $save);
  }

  /**
   * Get the parameters.
   *
   * @return array
   * @throws \Drupal\cloudconvert\Exceptions\InvalidParameterException
   */
  public function getParameters() {
    $parameterList = $this->parameterList;
    $parameters = [];
    foreach ($parameterList as $parameterName) {
      $parameterValue = $this->getParam($parameterName);
      if (is_null($parameterValue)) {
        continue;
      }
      $parameters[$parameterName] = $parameterValue;
    }

    return $parameters;
  }

}
