<?php

namespace Drupal\cloudconvert;

/**
 * CloudConvert Process Wrapper
 *
 * @package CloudConvert
 * @category CloudConvert
 * @author Josias Montag <josias@montag.info>
 */
class Process extends ApiObject {

  /**
   * The identifier for the process at CloudConvert.com
   *
   * @var string
   */
  protected $processId;

  /**
   * Construct a new Process instance
   *
   * @param Api $api
   * @param $processId
   * @param string $url The Process URL   *
   */
  public function __construct(Api $api, $processId, $url) {
    parent::__construct($api, $url);
    $this->processId = $processId;
    $this->refresh();
    return $this;
  }

  public function getProcessId() {
    return $this->processId;
  }

  /**
   * Starts the Process
   *
   * @param $file
   * @param $wait
   * @param array $parameters Parameters for creating the Process. See
   *   https://cloudconvert.com/apidoc#start
   *
   * @return \Drupal\cloudconvert\Process
   */
  public function start(Parameters $parameters = NULL) {
    $this->data = $this->api->post($this->url, $parameters->getParameters(), FALSE);
    return $this;
  }

  /**
   * Uploads the input file. See https://cloudconvert.com/apidoc#upload
   *
   * @param $stream
   * @param string $filename Filename of the input file
   *
   * @return \Drupal\cloudconvert\Process
   * @throws \Drupal\cloudconvert\Exceptions\ApiException
   */
  public function upload($stream, $filename = NULL) {
    if (!isset($this->upload->url)) {
      throw new Exceptions\ApiException("Upload is not allowed in this process state", 400);
    }

    if (empty($filename)) {
      $metadata = stream_get_meta_data($stream);
      $filename = basename($metadata['uri']);
    }
    $this->api->put($this->upload->url . "/" . rawurlencode($filename), $stream, FALSE);
    return $this;
  }

  /**
   * Waits for the Process to finish (or end with an error)
   *
   * @return \Drupal\cloudconvert\ApiObject
   */
  public function wait() {
    if ($this->step == 'finished' || $this->step == 'error') {
      return $this;
    }

    $parameters = new Parameters(['wait' => TRUE]);

    return $this->refresh($parameters);
  }

  /**
   * Download all files from cloudconvert.
   *
   * @param $destinationDirectory
   *
   * @return $this
   */
  public function downloadAllFiles($destinationDirectory) {
    $output = $this->get('output');
    foreach ($output->files as $file) {
      $this->downloadFile($destinationDirectory . '/' . $file, $output->url . '/' . $file);
    }

    return $this;
  }

  /**
   * Download a file from cloudconvert.
   *
   * @param $fileDestination
   * @param $outputUrl
   *
   * @return \Drupal\cloudconvert\Process
   */
  public function downloadFile($fileDestination, $outputUrl) {
    return $this->downloadStream(fopen($fileDestination, 'w'), $outputUrl);
  }

  /**
   * Set up a download stream with cloudconvert.
   *
   * @param $stream
   * @param $outputUrl
   *
   * @return $this
   */
  public function downloadStream($stream, $outputUrl) {
    $local = \GuzzleHttp\Psr7\stream_for($stream);
    $download = $this->api->get($outputUrl, FALSE, FALSE);
    $local->write($download);
    return $this;
  }

  /**
   * Delete Process from API
   *
   * @return \Drupal\cloudconvert\Process
   */
  public function delete() {
    $this->api->delete($this->url, FALSE, FALSE);
    return $this;
  }
}
